### load package for the model
library(gamboostLSS)
### load package for data
library(gamlss)
data("dbbmi")

### Graphical representation
pdf("hist_bmi.pdf", width = 9, height = 6)
par(mfrow=c(1,2))
### Histogram
hist(dbbmi$bmi, breaks=60, col="lightgrey", border="darkgrey",
     freq=FALSE, xlab = "BMI", main=""     )
legend(23,.18, lwd=2, col=c("lightgrey"), 
       legend = c("Data"),
       bty = "n")
### Histogram with Gaussian distribution based empirical parameters
hist(dbbmi$bmi, breaks=60, col="lightgrey", border="darkgrey",
     freq=FALSE, xlab = "BMI", main="")
lines(seq(-600,600,.1), dnorm(seq(-600,600,.1), mean(dbbmi$bmi), sd(dbbmi$bmi)), xlab="Z-Score",
      lwd=2)
legend("topright", lwd=2, col=c("lightgrey", "black"), 
       legend = c("Data", "Normal Distribution"),
       bty = "n")
dev.off()


pdf("hist_scatter.pdf", width = 9, height = 6)
par(mfrow=c(1,2))
### Histogram with Gaussian distribution based empirical parameters
hist(dbbmi$bmi, breaks=60, col="lightgrey", border="darkgrey",
     freq=FALSE, xlab = "BMI", main="")
lines(seq(-600,600,.1), dnorm(seq(-600,600,.1), mean(dbbmi$bmi), sd(dbbmi$bmi)), xlab="Z-Score",
      lwd=2)
legend("topright", lwd=2, col=c("lightgrey", "black"), 
       legend = c("Data", "Normal Distribution"),
       bty = "n")
plot(dbbmi$age, dbbmi$bmi, pch=16, col=rgb(.3,.3,.3,.2), 
     ylab = "BMI", xlab = "age", las = 1)
dev.off()

mean(dbbmi$bmi)
var(dbbmi$bmi)

### Comparison P(X>25)
1 - pnorm(30, mean(dbbmi$bmi), sqrt(var(dbbmi$bmi)))
length(which(dbbmi$bmi>30))/length(dbbmi$bmi)
### Comparison of 0.95 Quantile
sort(dbbmi$bmi)[round(.95*length(dbbmi$bmi))]
qnorm(.95, mean(dbbmi$bmi), sqrt(var(dbbmi$bmi)))

pdf("agebmi.pdf", width = 9, height = 6)
### Scatterplot of the data
plot(dbbmi$age, dbbmi$bmi, pch=16, col=rgb(.3,.3,.3,.2), 
     ylab = "BMI", xlab = "age", las = 1)
dev.off()


### Comparison to a nonlinear Gaussian model
### load package for mean regression
library(mgcv)
### build model with nonlinear effect for age
mod.gam <- gam(bmi ~ s(age, bs="ps"), data=dbbmi)
pdf("gamfit.pdf", width = 9, height = 6)
### scatterplot of the data
plot(dbbmi$age, dbbmi$bmi, pch = 16, col = rgb(.3,.3,.3,.1), 
     ylab = "BMI", xlab = "age", las = 1)
legend(0, 36, legend = c("observations",
                           "mean regression"),
       lty = c(0, 1), lwd = c(0, 3),
       pch = c(16, NA),
       col =  c(rgb(.3,.3,.3,.2),1),
       bty = "n")
### use order of covariate for plotting functions as lines
o <- order(dbbmi$age)
### add function of the nonlinear effect
lines(dbbmi$age[o],predict(mod.gam)[o],  lwd = 3)
dev.off()

### define vector of quantiles
pp = c(0.01, 0.1,0.2, 0.3,0.4,0.5,0.6,0.7,0.8,0.9, 0.99)
pdf("gamfitquant.pdf", width = 9, height = 6)
### scatterplot of the data
plot(dbbmi$age, dbbmi$bmi, pch = 16, col = rgb(.3,.3,.3,.1), 
     ylab = "BMI", xlab = "age", las = 1)
legend(0, 36, legend = c("observations",
                       "mean regression",
                       "derived quantiles"),
       lty = c(0, 1, 1), lwd = c(0, 3, 1),
       pch = c(16, NA, NA),
       col =  c(rgb(.3,.3,.3,.2),1,1),
       bty = "n")
### add function of the nonlinear effect
lines(dbbmi$age[o],predict(mod.gam)[o],  lwd = 3)
### calculcate quantiles of normal distribution
quant.norm <- qnorm(pp)
for(i in 1:11){
  ### calculate quantile with assumption of Gaussian distribution with model variance
  quant_est <- predict(mod.gam) + sqrt(mod.gam$sig2)*quant.norm[i]
  lines(dbbmi$age, quant_est)}
dev.off()

### define list to store boosting objects and vector for storing boostrap risks
qr_boost <- list()
cvrg <- list()
cvr <- vector(length=11)
for(i in 1:11){
  set.seed(1)
  ###store only predictions of the model (hence predict())
  ### Arguments in gamboost():
  ##### Arguments in formula(): 
  ####### first argument: dependent variable
  ####### age: continous variables 
  ##### family = QuantReg(tau = pp[i]): quantile regression for current value of pp[i]
  ##### data: dataset including all used covariates
  ##### Arguments in control():
  ####### mstop: number of iterations, determined via 25-fold boostrap (see below)
  ####### nu: step length, default = 0.1, needs to be to higher values for quantiles 
  #### first argument: the model for which the stopping iterations have to be determined
  #### grid: grid on which the stopping iterations have to be evaluated
  ### for more detailed explanations on this algorithm, see boosting tutorial paper:
  ### Boosting for statistical modelling -- a non-technical introduction
  ### Mayr, Andreas and Hofner, Benjamin
  ### Statistical Modelling
  ### 2017
  qr_boost_start <-  gamboost(bmi ~ age,
                             family = QuantReg(tau = pp[i]),
                             data = dbbmi,
                             control = boost_control(mstop=1, nu=0.5))
  cvrg[[i]]<- cvrisk(qr_boost_start, grid=1:10000)
  cvr[i] <- mstop(cvrg[[i]])
  qr_boost[[i]] <- predict(gamboost(bmi ~ age,
                           family = QuantReg(tau = pp[i]),
                           data = dbbmi,
                           control = boost_control(mstop=cvr[i], nu=0.5)))
  gc()
 }
## In this case the models do not really converge (due to having only one covariate):
cvr
## The risks do not increase (even when running the model on larger grid - change grid = 1:10000 to grid = 1:50000 to try)
## see the plotted risks (example: tau = 0.2, mstop = 10000)
plot(cvrg[[1]])

pdf("quantfit.pdf", width = 9, height = 6)
### scatterplot of the data
plot(dbbmi$age, dbbmi$bmi, pch = 16, col = rgb(.3,.3,.3,.1),
     ylab = "BMI", xlab = "age", las = 1)
### add lines of predictions of the models for each quantile
for(i in 1:11){
lines(dbbmi$age[o], qr_boost[[i]][o])}
legend(0, 36, legend = c("observations",
                          "quantiles"),
       lty = c(0,  1), lwd = c(0, 1),
       pch = c(16, NA),
       col =  c(rgb(.3,.3,.3,.2),1),
       bty = "n")
dev.off()

pdf("both.pdf", width = 9, height = 6)
par(mfrow=c(1,2))
plot(dbbmi$age, dbbmi$bmi, pch = 16, col = rgb(.3,.3,.3,.1), 
     ylab = "BMI", xlab = "age", las = 1)
legend(0, 36, legend = c(#"observations",
                         "mean regression",
                         "derived quantiles"),
       lty = c(#0,
               1, 1), lwd = c(#0, 
                              3, 1),
       # pch = c(16, NA, NA),
       col =  c(#rgb(.3,.3,.3,.2),
                1,1),
       bty = "n")
### add function of the nonlinear effect
lines(dbbmi$age[o],predict(mod.gam)[o],  lwd = 3)
### calculcate quantiles of normal distribution
quant.norm <- qnorm(pp)
for(i in 1:11){
  ### calculate quantile with assumption of Gaussian distribution with model variance
  quant_est <- predict(mod.gam) + sqrt(mod.gam$sig2)*quant.norm[i]
  lines(dbbmi$age, quant_est)}

plot(dbbmi$age, dbbmi$bmi, pch = 16, col = rgb(.3,.3,.3,.1),
     ylab = "BMI", xlab = "age", las = 1)
### add lines of predictions of the models for each quantile
for(i in 1:11){
  lines(dbbmi$age[o], qr_boost[[i]][o])}
legend(0, 36, legend = c(#"observations",
                         "quantiles"),
       lty = c(#0, 
               1), lwd = c(#0, 
                           1),
       # pch = c(16, NA),
       col =  c(#rgb(.3,.3,.3,.2),
                1),
       bty = "n")
dev.off()

### Compare the fit of the median regression to mean regression
pdf("compare_fit.pdf", width = 9, height = 6)
plot(dbbmi$age, dbbmi$bmi, pch = 16, col = rgb(.3,.3,.3,.1),
     ylab = "BMI", xlab = "age", las = 1)
lines(dbbmi$age[o], qr_boost[[6]][o], lwd = 3)
lines(dbbmi$age[o],predict(mod.gam)[o], lty = 2,  lwd = 3)

legend(0, 36, legend = c("observations",
                       "mean regression",
                       "median regression"),
       lty = c(0,  2, 1), lwd = c(0, 3, 3),
       pch = c(16, NA, NA),
       col =  c(rgb(.3,.3,.3,.2),1,1),
       bty = "n")
dev.off()

